/*

BBC Radio widget by Hawkman. http://www.phantomgorilla.com/
All suggestions and bug reports to hawkman@phantomgorilla.com please

*******************************************************************************

LICENCE

Free to use, free to look at and learn from, free to pinch small chunks of code from for use in your own projects.

However, you may NOT redistribute this widget, or a port or modified version of it, without obtaining permission from the author (via email, hawkman@phantomgorilla.com) and including credit to him in the finished project (a small notice in the widget's javascript/html files is all that's necessary; nothing visible on the widget).

This software is provided as-is, with no guarantee. The author can accept no responsibility for any consequences of its use.

*******************************************************************************

ABOUT THE WIDGET

- Basic Overview

This widget uses Real Player for the audio. Preferences are written upon the selection of an item in a pop-up menu, then this is compared to a list of stations to find the stream url. A function then writes html to the front of the widget to embed the realplayer plugin with the correct url, and changes the image source to reflect which station is playing.

- Credits

Basic Javascript routines taken out of Apple's sample Goodbye World widget. A small section of crash-preventing code is taken from Duncan Ponting's radio widget (http://www.bbc.co.uk/radio/prototypes/listenlive/) at his suggestion - cheers. Also taken from that widget is the idea to crawl the broswer plugins to find RealPlayer. Version string .plist lookup taken from Joshua Emmons' blank foo.wdgt (http://www.skia.net/). Study of his example AppleAnimator use was also extremely helpful. Some button-like routines adapted from Apple's GenericButton class.

Additional skills were obtained by studying Apple's sample files and the Mac OS X bundled widgets, however no actual code was used from these sources other than that mentioned already.

******************************************************************************/


/* 
Progress indicator by Hawkman. http://www.phantomgorilla.com/
based on  Apple's NSProgressIndicator in OS X. http://www.apple.com/
tested in Safari 2.0.3, FireFox (Mac) 1.0.7

FREELY AVAILABLE FROM:
http://www.phantomgorilla.com/20060411/nsprogressindicator-for-widgets/
*/


// Global variables, to be accessed from multiple functions
var progressIndicatorImages;
var index = 0;
var progressIndicatorTimer;


// Loads the images to be used in the progress indicator, and stores them in an
// array for easy access. We could contruct a method to call them later
// during animateProgressIndicator() instead, but this way we also load them
// before we actually need them, which prevents image flickering.
// 2 birds + 1 stone = :)
function loadProgressIndicatorImages(size)
{
    // set up the array to hold the images (12 of them, 0-11)
    progressIndicatorImages = new Array(12);
    // start a "for" loop, incrementing values of i between 0 and 11, to save
    // us some writing!
    for (var i=0;i < 12; i++)
    {
        // make a new image in progressIndicatorImages, the position of which
        // is determined by i (which increments by 1 each time, remember)
        progressIndicatorImages[i] = new Image();
        // set the source based on the value of i, generating references of the
        // type "images/indicator_small_3.png", for instance.
        // If you change the image names or locations you'll have to alter this
        // section accordingly.
        progressIndicatorImages[i].src = "images/indicator_"+size+"_"+(i)+".png";
    }
}


// Call this function to start the progress indicator spinning. It calls the
// function to load the images, and calls another function at intervals (to
// change the image). Requires passing of a size attribute, either "large" or
// "small", eg. calling startProgressIndicator("small")
function startProgressIndicator(size)
{
    // fades the radio station logo
    document.getElementById("fader").style.display="block";
    // calls the loadProgressIndicatorImages() function above
    // must also pass a size of indicator to use, "large" or "small"
    loadProgressIndicatorImages(size);
    // calls animateProgressIndicator() at intervals of 42 milliseconds
    // - that works out at 2 cycles/second, which is what Apple use in
    // NSProgressIndicator
    progressIndicatorTimer = setInterval('animateProgressIndicator()', 42);
}


// Call this function to stop the indicator. It clears the timer, hides it again
// and resets it to the start position
function stopProgressIndicator()
{
    // removes the fade from the radio station logo
    document.getElementById("fader").style.display="none";
    // clears the timer
	clearInterval(progressIndicatorTimer);
	// sets the timer to null, to provide an easy way of checking whether the
	// timer is running or not, ie.:
	// if (progressIndicatorTimer==null) :: timer is not running
	// if (progressIndicatorTimer!=null) :: timer is running
	// Useful if you want the same action to start/stop the indicator, just call
	// an intermediate function which checks the value of the timer
    progressIndicatorTimer=null;
    // hides the indicator
    document.getElementById("progress_indicator").style.display = "none";
    // resets the index global variable (it's not 0, despite our definition
    // above, because animateProgressIndicator() has been messing with it), so
    // the animation will start from the top next time as expected
    index=0;
    // if we moused over the radio station image while the indicator was running, we should be showing the "What's On?" link - but we suppressed it while the indicator was running to avoid confusion (see whatsOnOver()). Now, if appropriate, we must run whatsOnOver to fade the link in.
    if (shouldShowWhatsOn==true)
	{
	   // calls the fade in
	   whatsOnOver("");
    }
}


// This function does the actual animation, swapping the images in the indicator
// when it's called at intervals by startProgressIndicator(). No need to call
// this directly, it won't work in the same way.
function animateProgressIndicator()
{
    // change the source of the indicator image to another of those held in the
    // progressIndicatorImages array (determined by the current value of index)
    document.getElementById("progress_indicator").src = progressIndicatorImages[index].src;
    // set the indicator to block display (the CSS has it hidden to start with).
    // We call it here after the first image has been shown, because it saves us
    // a line explicitly resetting the indicator to the start position in
    // stopProgressIndicator(), as well as making it immaterial whether we call
    // the same size indicator each time
    document.getElementById("progress_indicator").style.display = "block";
    // if index is 11 - the last image - change it to 0, to start another cycle
    if (index==11)
    { index=0 }
    // otherwise, increment it by 1, so that the next image in sequence is shown
    // next time animateProgressIndicator() is called (in 42 milliseonds' time!)
    else
    { index=index+1 }
}